#include <amxmodx>
#include <fakemeta>
#include <reapi>
#include <revive_teammates>

const UNIT_SECOND = (1 << 12);
const FFADE_IN = 0x0000;
const FFADE_OUT = 0x0001;
const FFADE_STAYOUT = 0x0004;

enum (<<=1) {
	ALERT_CHAT = 1,
	ALERT_CENTER,
	ALERT_HUD
};

new const SECTION[] = "effects";

new g_sCvarPrefix[48],
	g_iCvarBar,
	g_iCvarObserver,
	g_iCvarLighting,
	g_sCvarAlert[4],
	g_sCvarRespawnSound[64],
	g_sCvarLootSound[64],
	g_sCvarLootBreakSound[64],
	g_iCvarReviveNotification,
	g_iCvarScreenFade,
	g_iCvarScreenFadeRespawn;

new g_Alert;

public plugin_init()
{
	register_plugin("RT: Effects", RT_VERSION_STR, "Emma Jule");
}

public RT_ParseConfigFile(JSON:jsonMain)
{
	new JSON:jsonEffects;
	
	if (json_object_has_value(jsonMain, SECTION, JSONObject))
	{
		jsonEffects = json_object_get_value(jsonMain, SECTION);
	}
	else
	{
		jsonEffects = json_init_object();
	}
	
	if (json_is_object(jsonEffects))
	{
		json_cvar_integer(jsonEffects, "bar", 1, "Процесс будет сопровождатся полоской по центру эркана", .cvar = g_iCvarBar);
		json_cvar_integer(jsonEffects, "observer", 1, "Камера автоматически перенаправит на игрока кто его использует", .cvar = g_iCvarObserver);
		json_cvar_integer(jsonEffects, "light", 0, "Подсветка трупа^nВнимание: может вызывать нагрузку из-за особенностей свечения движка!", .cvar = g_iCvarLighting);
		json_cvar_string(jsonEffects, "alert_mode", "c", "Режим уведомлений^na - в чат^nb - по центру экрана^nc - HUD (плавное сообщение)^nПримечание: можно использовать посовместно^n'' или 0 - Отключено", .cvar = g_sCvarAlert, .len = charsmax(g_sCvarAlert));
		json_cvar_string(jsonEffects, "respawn_sound", NULL_STRING, "Звук когда игрок возродился (вводите без sound/)", .cvar = g_sCvarRespawnSound, .len = charsmax(g_sCvarRespawnSound), .precache = true);
		json_cvar_string(jsonEffects, "loot_sound", NULL_STRING, "Звук когда начинаеш возрождать тиммейта (вводите без sound/)", .cvar = g_sCvarLootSound, .len = charsmax(g_sCvarLootSound), .precache = true);
		json_cvar_string(jsonEffects, "loot_break_sound", NULL_STRING, "Звук когда прекращено использование трупа (вводите без sound/)", .cvar = g_sCvarLootBreakSound, .len = charsmax(g_sCvarLootBreakSound), .precache = true);
		json_cvar_integer(jsonEffects, "revive_notification", 1, "Уведомление о воскрешении тиммейта^n0 - Нет^n1 - Всем^n2 - Только команде^n3 - Только трупу и тому кто его поднимал", .cvar = g_iCvarReviveNotification);
		json_cvar_integer(jsonEffects, "screenfade", 0, "Затемнение экрана в момент воскрешения игрока", .cvar = g_iCvarScreenFade);
		json_cvar_integer(jsonEffects, "screenfade_respawn", 0, "Затемнение экрана (блик) для воскрешенного игрока", .cvar = g_iCvarScreenFadeRespawn);
		
		json_object_set_value(jsonMain, SECTION, jsonEffects);
		json_free(jsonEffects);
	}
	else
		set_fail_state("Module <%s> loading failed. Check config settings!", SECTION);
}

public plugin_cfg()
{
	g_Alert = read_flags_ex(g_sCvarAlert);
	
	RT_GetPrefix(g_sCvarPrefix, charsmax(g_sCvarPrefix));
}

public RT_StartSneaking(const corpse_id, const owner, const Float:origin[3])
{
	if (g_iCvarLighting)
	{
		set_entvar(corpse_id, var_effects, get_entvar(corpse_id, var_effects) | EF_DIMLIGHT);
	}
}

public RT_StartLoot_Post(const corpse_id, const activator, const owner, Float:duration)
{
	if (RT_RelationShipStatus(owner, activator) != GR_TEAMMATE)
		return;
	
	if (g_iCvarObserver)
	{
		// set_entvar(owner, var_iuser2, OBS_IN_EYE);
		rg_internal_cmd(owner, "specmode", "4");
		set_entvar(owner, var_iuser2, activator);
		
		set_member(owner, m_hObserverTarget, activator);
		set_member(owner, m_flNextObserverInput, get_gametime() + 0.2);
	}
	
	if (g_iCvarBar && duration > 1.0)
	{
		rg_send_bartime(activator, floatround(duration));
	}
	
	if (g_Alert & ALERT_CHAT)
	{
		client_print_color(activator, owner, "%s %l", g_sCvarPrefix, "RT_PRINT_CHAT_REVIVE1", owner, duration);
		client_print_color(owner, activator, "%s %l", g_sCvarPrefix, "RT_PRINT_CHAT_REVIVE2", activator, duration);
	}
	
	if (g_Alert & ALERT_CENTER)
	{
		client_print(activator, print_center, "%l", "RT_PRINT_CENTER_REVIVE1", owner, duration);
		client_print(owner, print_center, "%l", "RT_PRINT_CENTER_REVIVE2", activator, duration);
	}
	
	if (g_Alert & ALERT_HUD)
	{
		set_dhudmessage(0, 160, 30, -1.0, 0.76, 2, 3.0, 2.0, 0.03, 0.4);
		show_dhudmessage(activator, "%l", "RT_PRINT_HUD_REVIVE1", owner, duration);
		
		set_dhudmessage(0, 160, 30, -1.0, 0.76, 2, 3.0, 2.0, 0.03, 0.4);
		show_dhudmessage(owner, "%l", "RT_PRINT_HUD_REVIVE2", activator, duration);
	}
	
	if (g_sCvarLootSound[0])
	{
		rh_emit_sound2(corpse_id, 0, CHAN_VOICE, g_sCvarLootSound);
	}
	
	if (g_iCvarScreenFade)
	{
		new color[3];
		switch (get_entvar(corpse_id, var_team))
		{
		case TEAM_TERRORIST:
			color = { 200, 0, 0 };
		case TEAM_CT:
			color = { 0, 0, 200 };
		}
		UTIL_ScreenFade(activator, 2.5, duration, color, 100, FFADE_OUT);
	}
}

public RT_StopLoot(const corpse_id, const activator, const owner, bool:complete)
{
	if (complete) {
		return;
	}

	if (RT_RelationShipStatus(owner, activator) != GR_TEAMMATE) {
		return;
	}
	
	if (g_iCvarBar)
	{
		rg_send_bartime(activator, 0);
	}
	
	if (g_iCvarScreenFade)
	{
		UTIL_ScreenFade(activator);
	}
	
	if (g_sCvarLootBreakSound[0])
	{
		rh_emit_sound2(corpse_id, 0, CHAN_VOICE, g_sCvarLootBreakSound);
	}
}

public RT_GotRespawn(const id, const friend)
{
	if (g_sCvarRespawnSound[0])
	{
		rh_emit_sound2(id, 0, CHAN_BODY, g_sCvarRespawnSound);
	}
	
	if (g_iCvarScreenFadeRespawn)
	{
		UTIL_ScreenFade(id, 1.0, 0.0, { 0, 200, 0 }, 125, FFADE_IN);
	}
	
	switch (g_iCvarReviveNotification)
	{
	case 1:
	{
		client_print_color(0, friend, "%s %l", g_sCvarPrefix, "RT_NOTIFICATION", friend, id);
	}
	case 2:
	{
		for (new i = MaxClients; i > 0; --i)
		{
			if (!is_user_connected(i))
				continue;
			
			if (RT_RelationShipStatus(i, friend) != GR_TEAMMATE)
				continue;
			
			client_print_color(i, print_team_default, "%s %l", g_sCvarPrefix, "RT_NOTIFICATION", friend, id);
		}
	}
	case 3:
	{
		client_print_color(id, print_team_default, "%s %l", g_sCvarPrefix, "RT_NOTIFICATION", friend, id);
		client_print_color(friend, print_team_default, "%s %l", g_sCvarPrefix, "RT_NOTIFICATION", friend, id);
	}
	}
}

stock UTIL_ScreenFade(const id, const Float:fxtime = 0.0, const Float:holdtime = 0.0, const color[3] = { 0, 0, 0 }, const alpha = 0, const flags = FFADE_IN)
{
	if (Float:get_member(id, m_blindUntilTime) > get_gametime())
		return;
	
	static msgScreenFade;
	if (msgScreenFade > 0 || (msgScreenFade = get_user_msgid("ScreenFade")) > 0)
	{
		message_begin(MSG_ONE_UNRELIABLE, msgScreenFade, .player = id);
		write_short(FixedUnsigned16(fxtime, UNIT_SECOND));
		write_short(FixedUnsigned16(holdtime, UNIT_SECOND));
		write_short(flags);
		write_byte(color[0]);
		write_byte(color[1]);
		write_byte(color[2]);
		write_byte(alpha);
		message_end();
	}
}

stock FixedUnsigned16(Float:value, scale = UNIT_SECOND)
{
	new output = floatround(value * scale);
	
	if (output < 0)
		output = 0;

	if (output > 0xFFFF)
		output = 0xFFFF;

	return output;
}